Package: Matrix
Version: 1.7-6
VersionNote: do also bump src/version.h, inst/include/Matrix/version.h
Date: 2026-07-24
Priority: recommended
Title: Sparse and Dense Matrix Classes and Methods
Description: A rich hierarchy of sparse and dense matrix classes,
	including general, symmetric, triangular, and diagonal matrices
	with numeric, logical, or pattern entries.  Efficient methods for
	operating on such matrices, often wrapping the 'BLAS', 'LAPACK',
	and 'SuiteSparse' libraries.
License: GPL (>= 2) | file LICENCE
URL: https://codeberg.org/jaganmn/Matrix
BugReports: https://codeberg.org/jaganmn/Matrix/issues
Contact: Matrix-authors@R-project.org
Authors@R: 
	c(person("Douglas", "Bates", role = "aut",
	         comment = c(ORCID = "0000-0001-8316-9503")),
	  person("Martin", "Maechler", role = c("aut", "cre"),
	         email = "mmaechler+Matrix@gmail.com",
	         comment = c(ORCID = "0000-0002-8685-9910")),
	  person("Mikael", "Jagan", role = "aut",
	         comment = c(ORCID = "0000-0002-3542-2938")),
	  person("Timothy A.", "Davis", role = "ctb",
	         comment = c(ORCID = "0000-0001-7614-6899",
	                     "SuiteSparse libraries",
	                     "collaborators listed in dir(system.file(\"doc\", \"SuiteSparse\", package=\"Matrix\"), pattern=\"License\", full.names=TRUE, recursive=TRUE)")),
	  person("George", "Karypis", role = "ctb",
	         comment = c(ORCID = "0000-0003-2753-1437",
	                     "METIS library",
	                     "Copyright: Regents of the University of Minnesota")),
	  person("Jason", "Riedy", role = "ctb",
	         comment = c(ORCID = "0000-0002-4345-4200",
	                     "GNU Octave's condest() and onenormest()",
	                     "Copyright: Regents of the University of California")),
	  person("Jens", "Oehlschlägel", role = "ctb",
	         comment = "initial nearPD()"),
	  person("R Core Team", role = "ctb",
	         comment = c(ROR = "02zz1nj61", "base R's matrix implementation")))
Depends: R (>= 4.4), methods
Imports: grDevices, graphics, grid, lattice, stats, utils
Suggests: MASS, datasets, sfsmisc, tools
Enhances: SparseM, graph
LazyData: no
LazyDataNote: not possible, since we use data/*.R and our S4 classes
BuildResaveData: no
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-07-24 11:40:35 UTC; maechler
Author: Douglas Bates [aut] (ORCID: <https://orcid.org/0000-0001-8316-9503>),
  Martin Maechler [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8685-9910>),
  Mikael Jagan [aut] (ORCID: <https://orcid.org/0000-0002-3542-2938>),
  Timothy A. Davis [ctb] (ORCID: <https://orcid.org/0000-0001-7614-6899>,
    SuiteSparse libraries, collaborators listed in
    dir(system.file("doc", "SuiteSparse", package="Matrix"),
    pattern="License", full.names=TRUE, recursive=TRUE)),
  George Karypis [ctb] (ORCID: <https://orcid.org/0000-0003-2753-1437>,
    METIS library, Copyright: Regents of the University of Minnesota),
  Jason Riedy [ctb] (ORCID: <https://orcid.org/0000-0002-4345-4200>, GNU
    Octave's condest() and onenormest(), Copyright: Regents of the
    University of California),
  Jens Oehlschlägel [ctb] (initial nearPD()),
  R Core Team [ctb] (ROR: <https://ror.org/02zz1nj61>, base R's matrix
    implementation)
Maintainer: Martin Maechler <mmaechler+Matrix@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-25 02:50:02 UTC
